@@ -103,10 +103,14 @@ sub number_of_captures {
     return $self->attributes->{CaptureArgs}[0] || 0;
 }
 
+sub scheme {
+  return exists $_[0]->attributes->{Scheme} ? $_[0]->attributes->{Scheme}[0] : undef;
+}
+
 sub list_extra_info {
   my $self = shift;
   return {
-    Args => $self->attributes->{Args}[0],
+    Args => exists $self->attributes->{Args} ? $self->attributes->{Args}[0] : undef,
     CaptureArgs => $self->number_of_captures,
   }
 } 
@@ -192,6 +196,10 @@ Returns the number of captures this action expects for L<Chained|Catalyst::Dispa
 
 A HashRef of key-values that an action can provide to a debugging screen
 
+=head2 scheme
+
+Any defined scheme for the action
+
 =head2 meta
 
 Provided by Moose.
