@@ -6,6 +6,7 @@ extends 'Catalyst::DispatchType';
 use Text::SimpleTable;
 use Catalyst::Utils;
 use URI;
+use Encode 2.21 'decode_utf8';
 
 has _paths => (
                is => 'rw',
@@ -57,10 +58,11 @@ sub list {
         foreach my $action ( @{ $self->_paths->{$path} } ) {
             my $args  = $action->attributes->{Args}->[0];
             my $parts = defined($args) ? '/*' x $args : '/...';
-            
+
             my $display_path = "/$path/$parts";
             $display_path =~ s{/{1,}}{/}g;
-             
+            $display_path =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg; # deconvert urlencoded for pretty view 
+            $display_path = decode_utf8 $display_path;  # URI does encoding
             $paths->row( $display_path, "/$action" );
         }
     }
